@@ -1,15 +1,6 @@
-"""SCons.Tool.gcc
-
-Tool-specific initialization for gcc.
-
-There normally shouldn't be any need to import this module directly.
-It will usually be imported through the generic SCons.Tool.Tool()
-selection method.
-
-"""
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -29,21 +20,27 @@ selection method.
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "src/engine/SCons/Tool/gcc.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"
+"""SCons.Tool.gcc
+
+Tool-specific initialization for gcc.
+
+There normally shouldn't be any need to import this module directly.
+It will usually be imported through the generic SCons.Tool.Tool()
+selection method.
+
+"""
 
 from . import cc
-import os
 import re
-import subprocess
+from subprocess import PIPE
 
 import SCons.Util
 
 compilers = ['gcc', 'cc']
 
 
-def generate(env):
+def generate(env) -> None:
     """Add Builders and construction variables for gcc to an Environment."""
 
     if 'CC' not in env:
@@ -51,7 +48,7 @@ def generate(env):
 
     cc.generate(env)
 
-    if env['PLATFORM'] in ['cygwin', 'msys', 'win32']:
+    if env['PLATFORM'] in ['cygwin', 'win32']:
         env['SHCCFLAGS'] = SCons.Util.CLVar('$CCFLAGS')
     else:
         env['SHCCFLAGS'] = SCons.Util.CLVar('$CCFLAGS -fPIC')
@@ -60,6 +57,10 @@ def generate(env):
     if version:
         env['CCVERSION'] = version
 
+    env['CCDEPFLAGS'] = '-MMD -MF ${TARGET}.d'
+    env["NINJA_DEPFILE_PARSE_FORMAT"] = 'gcc'
+
+
 
 def exists(env):
     # is executable, and is a GNU compiler (or accepts '--version' at least)
@@ -76,27 +77,23 @@ def detect_version(env, cc):
     # -dumpversion was added in GCC 3.0.  As long as we're supporting
     # GCC versions older than that, we should use --version and a
     # regular expression.
-    # pipe = SCons.Action._subproc(env, SCons.Util.CLVar(cc) + ['-dumpversion'],
-    pipe = SCons.Action._subproc(env, SCons.Util.CLVar(cc) + ['--version'],
-                                 stdin='devnull',
-                                 stderr='devnull',
-                                 stdout=subprocess.PIPE)
-    if pipe.wait() != 0:
+    # pipe = SCons.Action.scons_subproc_run(env, SCons.Util.CLVar(cc) + ['-dumpversion'],
+    cp = SCons.Action.scons_subproc_run(
+        env, SCons.Util.CLVar(cc) + ['--version'], stdout=PIPE
+    )
+    if cp.returncode:
         return version
 
-    with pipe.stdout:
-        # -dumpversion variant:
-        # line = pipe.stdout.read().strip()
-        # --version variant:
-        line = SCons.Util.to_str(pipe.stdout.readline())
-        # Non-GNU compiler's output (like AIX xlc's) may exceed the stdout buffer:
-        # So continue with reading to let the child process actually terminate.
-        while SCons.Util.to_str(pipe.stdout.readline()):
-            pass
+    # -dumpversion variant:
+    # line = cp.stdout.strip()
+    # --version variant:
+    try:
+        line = SCons.Util.to_str(cp.stdout.splitlines()[0])
+    except IndexError:
+        return version
 
     # -dumpversion variant:
-    # if line:
-    #     version = line
+    # version = line
     # --version variant:
     match = re.search(r'[0-9]+(\.[0-9]+)+', line)
     if match:
