@@ -1,15 +1,6 @@
-"""SCons.Tool.hpc++
-
-Tool-specific initialization for c++ on HP/UX.
-
-There normally shouldn't be any need to import this module directly.
-It will usually be imported through the generic SCons.Tool.Tool()
-selection method.
-
-"""
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -29,9 +20,15 @@ selection method.
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "src/engine/SCons/Tool/hpcxx.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"
+"""SCons.Tool.hpc++
+
+Tool-specific initialization for c++ on HP/UX.
+
+There normally shouldn't be any need to import this module directly.
+It will usually be imported through the generic SCons.Tool.Tool()
+selection method.
+"""
 
 import os.path
 
@@ -48,7 +45,7 @@ acc = None
 
 try:
     dirs = os.listdir('/opt')
-except (IOError, OSError):
+except OSError:
     # Not being able to read the directory because it doesn't exist
     # (IOError) or isn't readable (OSError) is okay.
     dirs = []
@@ -59,8 +56,8 @@ for dir in dirs:
         acc = cc
         break
 
-        
-def generate(env):
+
+def generate(env) -> None:
     """Add Builders and construction variables for g++ to an Environment."""
     cplusplus.generate(env)
 
@@ -70,10 +67,10 @@ def generate(env):
         # determine version of aCC
         with os.popen(acc + ' -V 2>&1') as p:
             line = p.readline().rstrip()
-        if line.find('aCC: HP ANSI C++') == 0:
+        if line.startswith('aCC: HP ANSI C++'):
             env['CXXVERSION'] = line.split()[-1]
 
-        if env['PLATFORM'] == 'cygwin' or env['PLATFORM'] == 'msys':
+        if env['PLATFORM'] == 'cygwin':
             env['SHCXXFLAGS'] = SCons.Util.CLVar('$CXXFLAGS')
         else:
             env['SHCXXFLAGS'] = SCons.Util.CLVar('$CXXFLAGS +Z')
