@@ -1,16 +1,6 @@
-"""SCons.Tool.icl
-
-Tool-specific initialization for the Intel C/C++ compiler.
-Supports Linux and Windows compilers, v7 and up.
-
-There normally shouldn't be any need to import this module directly.
-It will usually be imported through the generic SCons.Tool.Tool()
-selection method.
-
-"""
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,11 +20,21 @@ selection method.
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-from __future__ import division, print_function
 
-__revision__ = "/src/script/src/engine/SCons/Tool/intelc.py none 2019-12-17 02:06:27 none"
+"""Tool-specific initialization for the Intel C/C++ compiler.
 
-import math, sys, os.path, glob, string, re
+Supports Linux and Windows compilers, v7 and up.
+
+There normally shouldn't be any need to import this module directly.
+It will usually be imported through the generic SCons.Tool.Tool()
+selection method.
+"""
+
+import glob
+import math
+import os.path
+import re
+import sys
 
 is_windows = sys.platform == 'win32'
 is_win64 = is_windows and (os.environ['PROCESSOR_ARCHITECTURE'] == 'AMD64' or
@@ -175,9 +175,7 @@ def get_intel_registry_value(valuename, version=None, abi=None):
             except SCons.Util.RegError:
                 raise MissingRegistryError("%s was not found in the registry, for Intel compiler version %s, abi='%s'"%(K, version,abi))
 
-        except SCons.Util.RegError:
-            raise MissingRegistryError("%s was not found in the registry, for Intel compiler version %s, abi='%s'"%(K, version,abi))
-        except SCons.Util.WinError:
+        except (SCons.Util.RegError, OSError):
             raise MissingRegistryError("%s was not found in the registry, for Intel compiler version %s, abi='%s'"%(K, version,abi))
 
     # Get the value:
@@ -201,7 +199,7 @@ def get_all_compiler_versions():
         try:
             k = SCons.Util.RegOpenKeyEx(SCons.Util.HKEY_LOCAL_MACHINE,
                                         keyname)
-        except SCons.Util.WinError:
+        except OSError:
             # For version 13 or later, check for default instance UUID
             if is_win64:
                 keyname = 'Software\\WoW6432Node\\Intel\\Suites'
@@ -210,7 +208,7 @@ def get_all_compiler_versions():
             try:
                 k = SCons.Util.RegOpenKeyEx(SCons.Util.HKEY_LOCAL_MACHINE,
                                             keyname)
-            except SCons.Util.WinError:
+            except OSError:
                 return []
         i = 0
         versions = []
@@ -252,7 +250,7 @@ def get_all_compiler_versions():
                         print("scons: *** Ignoring "+str(value))
 
                 i = i + 1
-        except EnvironmentError:
+        except OSError:
             # no more subkeys
             pass
     elif is_linux or is_mac:
@@ -318,7 +316,7 @@ def get_intel_compiler_top(version, abi):
         if not os.path.exists(os.path.join(top, "Bin", "icl.exe")) \
               and not os.path.exists(os.path.join(top, "Bin", abi, "icl.exe")) \
               and not os.path.exists(os.path.join(top, "Bin", archdir, "icl.exe")):
-            raise MissingDirError("Can't find Intel compiler in %s"%(top))
+            raise MissingDirError("Can't find Intel compiler in %s" % top)
     elif is_mac or is_linux:
         def find_in_2008style_dir(version):
             # first dir is new (>=9.0) style, second is old (8.0) style.
@@ -386,16 +384,18 @@ def get_intel_compiler_top(version, abi):
     return top
 
 
-def generate(env, version=None, abi=None, topdir=None, verbose=0):
+def generate(env, version=None, abi=None, topdir=None, verbose: int=0):
     r"""Add Builders and construction variables for Intel C/C++ compiler
     to an Environment.
-    args:
-      version: (string) compiler version to use, like "80"
-      abi:     (string) 'win32' or whatever Itanium version wants
-      topdir:  (string) compiler top dir, like
-                         "c:\Program Files\Intel\Compiler70"
-                        If topdir is used, version and abi are ignored.
-      verbose: (int)    if >0, prints compiler version used.
+
+    Args:
+      version (str): compiler version to use, like "80"
+      abi (str): 'win32' or whatever Itanium version wants
+      topdir (str): directory containing compiler tree, e.g.
+        "c:\\Program Files\\Intel\\Compiler70".
+        If `topdir` is used, `version` and `abi` are ignored.
+      verbose: if >0, prints compiler version used.
+
     """
     if not (is_mac or is_linux or is_windows):
         # can't handle this platform
@@ -448,22 +448,28 @@ def generate(env, version=None, abi=None, topdir=None, verbose=0):
     if not topdir:
         # Normally this is an error, but it might not be if the compiler is
         # on $PATH and the user is importing their env.
-        class ICLTopDirWarning(SCons.Warnings.Warning):
+        class ICLTopDirWarning(SCons.Warnings.SConsWarning):
             pass
-        if (is_mac or is_linux) and not env.Detect('icc') or \
-           is_windows and not env.Detect('icl'):
 
+        if (
+            ((is_mac or is_linux) and not env.Detect('icc'))
+            or (is_windows and not env.Detect('icl'))
+        ):
             SCons.Warnings.enableWarningClass(ICLTopDirWarning)
-            SCons.Warnings.warn(ICLTopDirWarning,
-                                "Failed to find Intel compiler for version='%s', abi='%s'"%
-                                (str(version), str(abi)))
+            SCons.Warnings.warn(
+                ICLTopDirWarning,
+                "Failed to find Intel compiler for version='%s', abi='%s'"
+                % (str(version), str(abi)),
+            )
         else:
             # should be cleaned up to say what this other version is
             # since in this case we have some other Intel compiler installed
             SCons.Warnings.enableWarningClass(ICLTopDirWarning)
-            SCons.Warnings.warn(ICLTopDirWarning,
-                                "Can't find Intel compiler top dir for version='%s', abi='%s'"%
-                                    (str(version), str(abi)))
+            SCons.Warnings.warn(
+                ICLTopDirWarning,
+                "Can't find Intel compiler top dir for version='%s', abi='%s'"
+                % (str(version), str(abi)),
+            )
 
     if topdir:
         archdir={'x86_64': 'intel64',
@@ -495,15 +501,15 @@ def generate(env, version=None, abi=None, topdir=None, verbose=0):
                    'LIB'             : libdir,
                    'PATH'            : bindir,
                    'LD_LIBRARY_PATH' : libdir}
-            for p in list(paths.keys()):
-                env.PrependENVPath(p, os.path.join(topdir, paths[p]))
+            for p, v in paths.items():
+                env.PrependENVPath(p, os.path.join(topdir, v))
         if is_mac:
             paths={'INCLUDE'         : 'include',
                    'LIB'             : libdir,
                    'PATH'            : bindir,
                    'LD_LIBRARY_PATH' : libdir}
-            for p in list(paths.keys()):
-                env.PrependENVPath(p, os.path.join(topdir, paths[p]))
+            for p, v in paths.items():
+                env.PrependENVPath(p, os.path.join(topdir, v))
         if is_windows:
             #       env key    reg valname   default subdir of top
             paths=(('INCLUDE', 'IncludeDir', 'Include'),
@@ -563,20 +569,23 @@ def generate(env, version=None, abi=None, topdir=None, verbose=0):
         for ld in [envlicdir, reglicdir]:
             # If the string contains an '@', then assume it's a network
             # license (port@system) and good by definition.
-            if ld and (ld.find('@') != -1 or os.path.exists(ld)):
+            if ld and ('@' in ld or os.path.exists(ld)):
                 licdir = ld
                 break
         if not licdir:
             licdir = defaultlicdir
             if not os.path.exists(licdir):
-                class ICLLicenseDirWarning(SCons.Warnings.Warning):
+                class ICLLicenseDirWarning(SCons.Warnings.SConsWarning):
                     pass
                 SCons.Warnings.enableWarningClass(ICLLicenseDirWarning)
-                SCons.Warnings.warn(ICLLicenseDirWarning,
-                                    "Intel license dir was not found."
-                                    "  Tried using the INTEL_LICENSE_FILE environment variable (%s), the registry (%s) and the default path (%s)."
-                                    "  Using the default path as a last resort."
-                                        % (envlicdir, reglicdir, defaultlicdir))
+                SCons.Warnings.warn(
+                    ICLLicenseDirWarning,
+                    "Intel license dir was not found.  "
+                    "Tried using the INTEL_LICENSE_FILE environment variable "
+                    "(%s), the registry (%s) and the default path (%s).  "
+                    "Using the default path as a last resort."
+                    % (envlicdir, reglicdir, defaultlicdir)
+                )
         env['ENV']['INTEL_LICENSE_FILE'] = licdir
 
 def exists(env):
