@@ -1,9 +1,9 @@
-__version__="4.10.0"
+__version__="4.10.1"
 __copyright__="Copyright (c) 2001 - 2025 The SCons Foundation"
 __developer__="bdbaddog"
-__date__="Thu, 02 Oct 2025 10:16:02 -0700"
+__date__="Sun, 16 Nov 2025 14:18:20 -0700"
 __buildsys__="M1Dog2021"
-__revision__="b3e979a3e42c43b91f7624895910afb869a19806"
-__build__="b3e979a3e42c43b91f7624895910afb869a19806"
+__revision__="055b01f429d58b686701a56df863a817c36bb103"
+__build__="055b01f429d58b686701a56df863a817c36bb103"
 # make sure compatibility is always in place
 import SCons.compat  # noqa
\ No newline at end of file
